<?php

namespace Teknisa\CRM\Controller;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Response\Error;
use Doctrine\ORM\EntityManager;
use Teknisa\CRM\Service\CRMAssistentesService;
use Zeedhi\Framework\DataSource\DataSet;

class CRMAssistentesController extends TeknisaCrud {
    protected $connection;
    protected $assistService;
    
    public function __construct(EntityManager $entityManager, CRMAssistentesService $assistService) {
        $this->connection = $entityManager->getConnection();
        $this->assistService = $assistService;
    }
    
    public function saveNewAssist(Request\Row $request, Response $response) {
        $row = $request->getRow();
        
        $this->connection->beginTransaction();
        try {
            
            $nrAssistencias = array();
            
            $added = array();
            $notAdded = array();
            $errors = array();
            
            foreach($row["NROPERASSIST"] as $key => $nroperassist) {
                $operador = array(
                    "NMOPERADOR"    => $row["NMOPERASSIST"][$key],
                    "NRPARCNEGOCIO" => $nroperassist
                );
                
                try{
                    $nrassistencia = $this->assistService->saveNewAssist($row, $nroperassist);
                    
                    $relationAssistNrParc = $nroperassist . "=>" . $nrassistencia;
                    array_push($nrAssistencias, $relationAssistNrParc);
                    
                    array_push($added, $operador);
                } catch(\Exception $e) {
                    array_push($notAdded, $operador);
                    array_push($errors, $e->getMessage());
                }
            }
            
            $returnArray = array(
                "NRASSISTENCIA" => $nrAssistencias,
                "ADDED"         => $added,
                "NOT_ADDED"     => $notAdded,
                "ERRORS"        => $errors
            );
            
            $response->addDataSet(new DataSet("DADOS", $returnArray));
            $this->connection->commit();
        } catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
            $this->connection->rollBack();
        }
    }
    
    public function removeAssist(Request\Row $request, Response $response) {
        $row = $request->getRow();
        
        $this->connection->beginTransaction();
        try {
            
            
            $removed    = array();
            $notRemoved = array();
            $errors     = array();
            
            foreach($row["ASSISTENCIAS"] as $key => $assistencia) {
                $operador = array(
                    "NMOPERADOR"    => $assistencia["NMOPERADOR"],
                    "NRPARCNEGOCIO" => $assistencia["NRPARCNEGOCIO"]
                );
                
                try {
                    
                    $this->assistService->removeAssist($assistencia["NRASSISTENCIA"]);
                    array_push($removed, $operador);
                } catch(\Exception $e) {
                    array_push($notRemoved, $operador);
                    array_push($errors, $e->getMessage());
                }
            }
            
            $returnArray = array(
                "REMOVED"         => $removed,
                "NOT_REMOVED"     => $notRemoved,
                "ERRORS"          => $errors
            );
            
            $response->addDataSet(new DataSet("DADOS", $returnArray));
            $this->connection->commit();
        } catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
            $this->connection->rollBack();
        }
    }
    
    public function checkIfCanEditOrEvolParc(Request\Row $request, Response $response) {
        $row = $request->getRow();
        $canEditOrEvol = true;
        
        try {
            $paramIsActive = $this->assistService->checkParamEditParcIsActive();
            if($paramIsActive) {
                $canEditOrEvol = $this->assistService->checkIfCanEditOrEvolParc($row);
            } else {
                $canEditOrEvol = true;
            }
            
            $returnArray = array(
                "CAN_EDIT" => $canEditOrEvol
            );
            
            $response->addDataSet(new DataSet("DADOS", $returnArray));
        } catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
            
    }
}