/*  PME - Planejamento de Merenda

Nome: ATUALIZA_COMENSAL

Descrio:
    Atualiza o nmero de comensais do cardpio a partir da
    tela de elaborao do cardpio.
    Caso o cardpio for especfico, sero atualizados os comensais
    somente daquele cardpio, mas se for um cardpio da central,
    a partir de uma lista a ser informada na tela, o operador informar
    quais as filiais que tero os comensais atualizados.

    P_CDFILCENTPME - Central de Merenda
    P_NRCARDPME    - Nmero do Cardpio
    P_NRLISTA      - N da Lista que contm as Filiais Agregadas
    P_CDSERVFILI   - Servio
    P_CDTIPOCARD   - Tipo de Cardpio
    P_DTINI        - Data inicial.
    P_DTFIN        - Data final.

Criao:
    Matheus Totte - 08/11/2006
*/

CREATE OR REPLACE PROCEDURE ATUALIZA_COMENSAL
     (P_CDFILCENTPME IN FILIAL.CDFILCENTPME   %TYPE,
      P_NRCARDPME    IN CARDAPIOPME.NRCARDPME %TYPE,
      P_NRLISTA      IN LISTA.NRLISTA         %TYPE,
      P_CDSERVFILI   IN SERVICO.CDSERVFILI    %TYPE,
      P_CDTIPOCARD   IN TIPOCARD.CDTIPOCARD   %TYPE,
      P_DTINI        IN DATE,
      P_DTFIN        IN DATE)

AS
   V_QTCOMPMESEG FLOAT;    V_QTCLIPMESEG FLOAT;
   V_QTCOMPMETER FLOAT;    V_QTCLIPMETER FLOAT;
   V_QTCOMPMEQUA FLOAT;    V_QTCLIPMEQUA FLOAT;
   V_QTCOMPMEQUI FLOAT;    V_QTCLIPMEQUI FLOAT;
   V_QTCOMPMESEX FLOAT;    V_QTCLIPMESEX FLOAT;
   V_QTCOMPMESAB FLOAT;    V_QTCLIPMESAB FLOAT;
   V_QTCOMPMEDOM FLOAT;    V_QTCLIPMEDOM FLOAT;
   V_QTCOMPMEFER FLOAT;    V_QTCLIPMEFER FLOAT;

   V_QTFUNPMESEG FLOAT;    V_QTDOAPMESEG FLOAT;
   V_QTFUNPMETER FLOAT;    V_QTDOAPMETER FLOAT;
   V_QTFUNPMEQUA FLOAT;    V_QTDOAPMEQUA FLOAT;
   V_QTFUNPMEQUI FLOAT;    V_QTDOAPMEQUI FLOAT;
   V_QTFUNPMESEX FLOAT;    V_QTDOAPMESEX FLOAT;
   V_QTFUNPMESAB FLOAT;    V_QTDOAPMESAB FLOAT;
   V_QTFUNPMEDOM FLOAT;    V_QTDOAPMEDOM FLOAT;
   V_QTFUNPMEFER FLOAT;    V_QTDOAPMEFER FLOAT;

   V_CDFILIAL_AUX   VARCHAR2(04);
   V_DIASEMANA      NUMBER;

BEGIN
   V_CDFILIAL_AUX := 'XXXX';

   FOR C_CARDAPIO IN (SELECT
                         CD.CDFILCENTPME, CD.NRCARDPME,   CD.NRCARDIAPME,  CD.CDSERVFILI,
                         CD.CDFILIAL,     CD.DTCARDIAPME, CD.QTCOMESTCPME, CP.CDTIPOCARD
                      FROM
                         CARDIAPME CD, CARDAPIOPME CP, LISTA LT
                      WHERE
                         (CD.CDFILCENTPME = P_CDFILCENTPME ) AND
                         (CD.NRCARDPME    = P_NRCARDPME    ) AND
                         (CD.CDSERVFILI   = P_CDSERVFILI   ) AND
                         (LT.NRLISTA      = P_NRLISTA      ) AND
                         (CD.CDFILIAL     = LT.CDLISTA     ) AND
                         (CP.CDFILCENTPME = CD.CDFILCENTPME) AND
                         (CP.NRCARDPME    = CD.NRCARDPME   ) AND
                         (CP.CDTIPOCARD   = P_CDTIPOCARD   ) AND
                         (CD.DTCARDIAPME BETWEEN P_DTINI AND P_DTFIN)
                      ORDER BY
                         CDFILIAL) LOOP

      IF (V_CDFILIAL_AUX <> C_CARDAPIO.CDFILIAL) THEN
         BEGIN
            SELECT
               NVL(QTCOMPMESEG, 0), NVL(QTCOMPMETER, 0), NVL(QTCOMPMEQUA, 0), NVL(QTCOMPMEQUI, 0),
               NVL(QTCOMPMESEX, 0), NVL(QTCOMPMESAB, 0), NVL(QTCOMPMEDOM, 0), NVL(QTCOMPMEFER, 0),
               NVL(QTCLIPMESEG, 0), NVL(QTCLIPMETER, 0), NVL(QTCLIPMEQUA, 0), NVL(QTCLIPMEQUI, 0),
               NVL(QTCLIPMESEX, 0), NVL(QTCLIPMESAB, 0), NVL(QTCLIPMEDOM, 0), NVL(QTCLIPMEFER, 0),
               NVL(QTFUNPMESEG, 0), NVL(QTFUNPMETER, 0), NVL(QTFUNPMEQUA, 0), NVL(QTFUNPMEQUI, 0),
               NVL(QTFUNPMESEX, 0), NVL(QTFUNPMESAB, 0), NVL(QTFUNPMEDOM, 0), NVL(QTFUNPMEFER, 0),
               NVL(QTDOAPMESEG, 0), NVL(QTDOAPMETER, 0), NVL(QTDOAPMEQUA, 0), NVL(QTDOAPMEQUI, 0),
               NVL(QTDOAPMESEX, 0), NVL(QTDOAPMESAB, 0), NVL(QTDOAPMEDOM, 0), NVL(QTDOAPMEFER, 0)
            INTO
               V_QTCOMPMESEG, V_QTCOMPMETER, V_QTCOMPMEQUA, V_QTCOMPMEQUI,
               V_QTCOMPMESEX, V_QTCOMPMESAB, V_QTCOMPMEDOM, V_QTCOMPMEFER,
               V_QTCLIPMESEG, V_QTCLIPMETER, V_QTCLIPMEQUA, V_QTCLIPMEQUI,
               V_QTCLIPMESEX, V_QTCLIPMESAB, V_QTCLIPMEDOM, V_QTCLIPMEFER,
               V_QTFUNPMESEG, V_QTFUNPMETER, V_QTFUNPMEQUA, V_QTFUNPMEQUI,
               V_QTFUNPMESEX, V_QTFUNPMESAB, V_QTFUNPMEDOM, V_QTFUNPMEFER,
               V_QTDOAPMESEG, V_QTDOAPMETER, V_QTDOAPMEQUA, V_QTDOAPMEQUI,
               V_QTDOAPMESEX, V_QTDOAPMESAB, V_QTDOAPMEDOM, V_QTDOAPMEFER
            FROM
               COMESERVTCAR
            WHERE
               (CDFILCENTPME = C_CARDAPIO.CDFILCENTPME) AND
               (CDTIPOCARD   = C_CARDAPIO.CDTIPOCARD  ) AND
               (CDSERVFILI   = C_CARDAPIO.CDSERVFILI  ) AND
               (CDFILIAL     = C_CARDAPIO.CDFILIAL    );
         EXCEPTION
            WHEN NO_DATA_FOUND THEN BEGIN
               V_QTCOMPMESEG := 0; V_QTCOMPMETER := 0; V_QTCOMPMEQUA := 0; V_QTCOMPMEQUI := 0;
               V_QTCLIPMESEG := 0; V_QTCLIPMETER := 0; V_QTCLIPMEQUA := 0; V_QTCLIPMEQUI := 0;
               V_QTFUNPMESEG := 0; V_QTFUNPMETER := 0; V_QTFUNPMEQUA := 0; V_QTFUNPMEQUI := 0;
               V_QTDOAPMESEG := 0; V_QTDOAPMETER := 0; V_QTDOAPMEQUA := 0; V_QTDOAPMEQUI := 0;
               V_QTCOMPMESEX := 0; V_QTCOMPMESAB := 0; V_QTCOMPMEDOM := 0; V_QTCOMPMEFER := 0;
               V_QTCLIPMESEX := 0; V_QTCLIPMESAB := 0; V_QTCLIPMEDOM := 0; V_QTCLIPMEFER := 0;
               V_QTFUNPMESEX := 0; V_QTFUNPMESAB := 0; V_QTFUNPMEDOM := 0; V_QTFUNPMEFER := 0;
               V_QTDOAPMESEX := 0; V_QTDOAPMESAB := 0; V_QTDOAPMEDOM := 0; V_QTDOAPMEFER := 0;
            END;
         END;

         V_CDFILIAL_AUX := C_CARDAPIO.CDFILIAL;
      END IF;

      BEGIN
         SELECT 0
         INTO   V_DIASEMANA
         FROM   FERIFILI
         WHERE  (CDFILIAL   = C_CARDAPIO.CDFILIAL   ) AND
                (DTFERIFILI = C_CARDAPIO.DTCARDIAPME);
      EXCEPTION
         WHEN NO_DATA_FOUND THEN
            V_DIASEMANA := TO_CHAR(C_CARDAPIO.DTCARDIAPME, 'D');
      END;

      UPDATE CARDIAPME
      SET
         QTCOMESTCPME = DECODE(V_DIASEMANA, 0, V_QTCLIPMEFER,
                             DECODE(V_DIASEMANA, 1, V_QTCLIPMEDOM,
                                  DECODE(V_DIASEMANA, 2, V_QTCLIPMESEG,
                                       DECODE(V_DIASEMANA, 3, V_QTCLIPMETER,
                                            DECODE(V_DIASEMANA, 4, V_QTCLIPMEQUA,
                                                 DECODE(V_DIASEMANA, 5, V_QTCLIPMEQUI,
                                                      DECODE(V_DIASEMANA, 6, V_QTCLIPMESEX,
                                                           V_QTCLIPMESAB))))))) +
                        DECODE(V_DIASEMANA, 0, V_QTFUNPMEFER,
                             DECODE(V_DIASEMANA, 1, V_QTFUNPMEDOM,
                                  DECODE(V_DIASEMANA, 2, V_QTFUNPMESEG,
                                       DECODE(V_DIASEMANA, 3, V_QTFUNPMETER,
                                            DECODE(V_DIASEMANA, 4, V_QTFUNPMEQUA,
                                                 DECODE(V_DIASEMANA, 5, V_QTFUNPMEQUI,
                                                      DECODE(V_DIASEMANA, 6, V_QTFUNPMESEX,
                                                           V_QTFUNPMESAB))))))) +
                        DECODE(V_DIASEMANA, 0, V_QTDOAPMEFER,
                             DECODE(V_DIASEMANA, 1, V_QTDOAPMEDOM,
                                  DECODE(V_DIASEMANA, 2, V_QTDOAPMESEG,
                                       DECODE(V_DIASEMANA, 3, V_QTDOAPMETER,
                                            DECODE(V_DIASEMANA, 4, V_QTDOAPMEQUA,
                                                 DECODE(V_DIASEMANA, 5, V_QTDOAPMEQUI,
                                                      DECODE(V_DIASEMANA, 6, V_QTDOAPMESEX,
                                                           V_QTDOAPMESAB))))))),

         QTCLIESTCPME = DECODE(V_DIASEMANA, 0, V_QTCLIPMEFER,
                             DECODE(V_DIASEMANA, 1, V_QTCLIPMEDOM,
                                  DECODE(V_DIASEMANA, 2, V_QTCLIPMESEG,
                                       DECODE(V_DIASEMANA, 3, V_QTCLIPMETER,
                                            DECODE(V_DIASEMANA, 4, V_QTCLIPMEQUA,
                                                 DECODE(V_DIASEMANA, 5, V_QTCLIPMEQUI,
                                                      DECODE(V_DIASEMANA, 6, V_QTCLIPMESEX,
                                                           V_QTCLIPMESAB))))))),

         QTFUNESTCPME = DECODE(V_DIASEMANA, 0, V_QTFUNPMEFER,
                             DECODE(V_DIASEMANA, 1, V_QTFUNPMEDOM,
                                  DECODE(V_DIASEMANA, 2, V_QTFUNPMESEG,
                                       DECODE(V_DIASEMANA, 3, V_QTFUNPMETER,
                                            DECODE(V_DIASEMANA, 4, V_QTFUNPMEQUA,
                                                 DECODE(V_DIASEMANA, 5, V_QTFUNPMEQUI,
                                                      DECODE(V_DIASEMANA, 6, V_QTFUNPMESEX,
                                                           V_QTFUNPMESAB))))))),

         QTDOAESTCPME = DECODE(V_DIASEMANA, 0, V_QTDOAPMEFER,
                             DECODE(V_DIASEMANA, 1, V_QTDOAPMEDOM,
                                  DECODE(V_DIASEMANA, 2, V_QTDOAPMESEG,
                                       DECODE(V_DIASEMANA, 3, V_QTDOAPMETER,
                                            DECODE(V_DIASEMANA, 4, V_QTDOAPMEQUA,
                                                 DECODE(V_DIASEMANA, 5, V_QTDOAPMEQUI,
                                                      DECODE(V_DIASEMANA, 6, V_QTDOAPMESEX,
                                                           V_QTDOAPMESAB)))))))
      WHERE
         (CDFILCENTPME = C_CARDAPIO.CDFILCENTPME) AND
         (NRCARDPME    = C_CARDAPIO.NRCARDPME   ) AND
         (CDSERVFILI   = C_CARDAPIO.CDSERVFILI  ) AND
         (CDFILIAL     = C_CARDAPIO.CDFILIAL    ) AND
         (DTCARDIAPME  = C_CARDAPIO.DTCARDIAPME );
   END LOOP;

   UPDATE CARDIAPME CA
   SET
      QTCOMESTCPME = (SELECT SUM(CD.QTCOMESTCPME)
                      FROM   CARDIAPME CD, CARDAPIOPME CP
                      WHERE
                         (CD.CDFILCENTPME = CA.CDFILCENTPME) AND
                         (CD.NRCARDPME    = CA.NRCARDPME   ) AND
                         (CD.CDSERVFILI   = CA.CDSERVFILI  ) AND
                         (CD.CDFILIAL     <> P_CDFILCENTPME) AND
                         (CP.CDFILCENTPME = CD.CDFILCENTPME) AND
                         (CP.NRCARDPME    = CD.NRCARDPME   ) AND
                         (CP.CDTIPOCARD   = P_CDTIPOCARD   ) AND
                         (CD.DTCARDIAPME  = CA.DTCARDIAPME ) AND
                         (CD.IDCARDESPEC  <> 'S')),

      QTCLIESTCPME = (SELECT SUM(CD.QTCLIESTCPME)
                      FROM   CARDIAPME CD, CARDAPIOPME CP
                      WHERE
                         (CD.CDFILCENTPME = CA.CDFILCENTPME) AND
                         (CD.NRCARDPME    = CA.NRCARDPME   ) AND
                         (CD.CDSERVFILI   = CA.CDSERVFILI  ) AND
                         (CD.CDFILIAL     <> P_CDFILCENTPME) AND
                         (CP.CDFILCENTPME = CD.CDFILCENTPME) AND
                         (CP.NRCARDPME    = CD.NRCARDPME   ) AND
                         (CP.CDTIPOCARD   = P_CDTIPOCARD   ) AND
                         (CD.DTCARDIAPME  = CA.DTCARDIAPME ) AND
                         (CD.IDCARDESPEC  <> 'S')),

      QTFUNESTCPME = (SELECT SUM(CD.QTFUNESTCPME)
                      FROM   CARDIAPME CD, CARDAPIOPME CP
                      WHERE
                         (CD.CDFILCENTPME = CA.CDFILCENTPME) AND
                         (CD.NRCARDPME    = CA.NRCARDPME   ) AND
                         (CD.CDSERVFILI   = CA.CDSERVFILI  ) AND
                         (CD.CDFILIAL     <> P_CDFILCENTPME) AND
                         (CP.CDFILCENTPME = CD.CDFILCENTPME) AND
                         (CP.NRCARDPME    = CD.NRCARDPME   ) AND
                         (CP.CDTIPOCARD   = P_CDTIPOCARD   ) AND
                         (CD.DTCARDIAPME  = CA.DTCARDIAPME ) AND
                         (CD.IDCARDESPEC  <> 'S')),

      QTDOAESTCPME = (SELECT SUM(CD.QTDOAESTCPME)
                      FROM   CARDIAPME CD, CARDAPIOPME CP
                      WHERE
                         (CD.CDFILCENTPME = CA.CDFILCENTPME) AND
                         (CD.NRCARDPME    = CA.NRCARDPME   ) AND
                         (CD.CDSERVFILI   = CA.CDSERVFILI  ) AND
                         (CD.CDFILIAL     <> P_CDFILCENTPME) AND
                         (CP.CDFILCENTPME = CD.CDFILCENTPME) AND
                         (CP.NRCARDPME    = CD.NRCARDPME   ) AND
                         (CP.CDTIPOCARD   = P_CDTIPOCARD   ) AND
                         (CD.DTCARDIAPME  = CA.DTCARDIAPME ) AND
                         (CD.IDCARDESPEC  <> 'S'))
   WHERE
      (CA.CDFILCENTPME = P_CDFILCENTPME           ) AND
      (CA.NRCARDPME    = P_NRCARDPME              ) AND
      (CA.CDSERVFILI   = P_CDSERVFILI             ) AND
      (CA.CDFILIAL     = CA.CDFILCENTPME          ) AND
      (CA.DTCARDIAPME  BETWEEN P_DTINI AND P_DTFIN);

   DELETE FROM LISTA
   WHERE (NRLISTA = P_NRLISTA);
END;
/

-- VERSAO --
BEGIN ATUVEROBJ('ATUALIZA_COMENSAL','5.05.015'); END;
/
