/* global UtilDevTools, Configuration */
(function(){
    function JsonTreeService(ElementToMetadataService){
        var self = this;
        var lastItem;
        var treeItems = [];
        var jsonUnloadedItems;

        this.setUnloadedItems = function(unloadedItems){
            jsonUnloadedItems = unloadedItems;
        };

        this.getUnloadedItems = function(){
            return jsonUnloadedItems;
        };

        this.setLastItem = function(item){
            lastItem = item;
        };

        this.getLastItem = function(){
            return lastItem;
        };

        var setTreeItems = function(items){
            treeItems = items;
        };

        this.getTreeItems = function(){
            return treeItems;
        };

        this.getItemMetadata = function(item){
            var parent = self.getTreeItems()[0];
            return ElementToMetadataService.getElementToMetadata(item, parent);
        };

        this.openTree = function(pathToOpen, lastItem){
            if(pathToOpen.length >= 2 && lastItem){
                var type = pathToOpen.shift();
                var index = pathToOpen.shift();
                lastItem._opened = true;
                if(lastItem[type][index]){
                    lastItem = lastItem[type][index];
                    return self.openTree(pathToOpen, lastItem);
                }
            }
            return lastItem;
        };

        this.loadTree = function(newContainer){
            var tree = [];
            if (newContainer){
                tree.push(newContainer);
            }
            setTreeItems(tree);
        };
        this.updateTree = self.loadTree;

        this.getDefaultTreeItem = function(){
            return window.currentContainer;
        };

        this.getTotalUnloadedItems = function(container){
            var childrenMapProperties = {
                container: ["widgets"],
                widgets: ["widgets", "fields", "actions"]
            };
            var hasSomeUnloadedItems = !container._element;
            var getTotalItems = function(item, type, initValue){
                var total = initValue;
                if(childrenMapProperties[type]){
                    total += childrenMapProperties[type].reduce(function(total, property){
                        var children = item[property];
                        if(children && children.length){
                            total += children.reduce(function(total, child){
                                if(child.showInDevTools){
                                    total++;
                                    hasSomeUnloadedItems = hasSomeUnloadedItems || !child._element;
                                    return total + getTotalItems(child, property, 0);
                                } else {
                                    return total;
                                }
                            }, 0);
                        }
                        return total;
                    }, 0);
                }
                return total;
            };
            var totalUnloadedItems = getTotalItems(container, "container", 1);
            totalUnloadedItems = (hasSomeUnloadedItems || window.opener) ? totalUnloadedItems : 0;
            return totalUnloadedItems;
        };
    }

    Configuration(function(DeveloperTreeService, ElementToMetadataService){
        var instance = new JsonTreeService(ElementToMetadataService);
        DeveloperTreeService.insertTreeMode('json', instance);
        DeveloperTreeService.insertTreeMode('form', instance); //@toDo transformar o form em apenas modo do editor
    });
})();
